from curl_http import HTTP
import lxml.html
import json
import xmltodict
import re
import random


class BaseAPI(HTTP):
    def __init__(self):
        super(BaseAPI, self).__init__()  # 执行父类初始化方法
        user_agents = [
            "User-Agent,Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50",
            "User-Agent,Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50",
            "User-Agent, Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv,2.0.1) Gecko/20100101 Firefox/4.0.1",
            "User-Agent,Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0;",
            "User-Agent, Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)",
            "User-Agent,Mozilla/5.0 (Windows NT 6.1; rv,2.0.1) Gecko/20100101 Firefox/4.0.1",
            "User-Agent, Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; 360SE)",
        ]
        self.set_user_agent(random.choice(user_agents))

    def get_table(self, doc):
        root = lxml.html.fromstring(doc)
        tables = []
        for tbl in root.xpath('//table'):
            elements = tbl.xpath('.//tr')  # 找到所以的tr
            table = []
            if len(elements) > 1:
                index = self.get_td_text(elements[0])
                for element in elements[1:]:
                    value = self.get_td_text(element)
                    table.append(dict(zip(index, value)))
            else:
                table.append(self.get_td_text(elements[0]))
            tables.append(table)
        return tables

    def get_td_text(self,element):
        tds = element.xpath("./td")
        value = []
        for td in tds:
            value.append(td.text_content())
        return value

    def get_stable(self, doc):
        root = lxml.html.fromstring(doc)
        result = [
            {'classes': ['', '', '', '', '', '', ''], 'time': '1 - 2节'},
            {'classes': ['', '', '', '', '', '', ''], 'time': '3 - 4节'},
            {'classes': ['', '', '', '', '', '', ''], 'time': '5 - 6节'},
            {'classes': ['', '', '', '', '', '', ''], 'time': '7 - 8节'},
            {'classes': ['', '', '', '', '', '', ''], 'time': '9 - 10节'},
            {'classes': ['', '', '', '', '', '', ''], 'time': '11 - 12节'},
        ]
        for i in range(1, 8):
            for j in range(1, 6):
                xpath = '//*[@id="k{}{}"]'.format(i, j)  # 得到表格
                td = root.xpath(xpath)[0]
                divs = td.xpath("././span/div")
                for div in divs:
                    temp = div.xpath(".//text()")
                    name = temp[0]
                    info = temp[1].split("  ")
                    course = self.get_course_info(info)
                    course["status"] = info[2]  # 是否选中
                    notes = info[1].split(" ")
                    if len(notes) > 1:
                        course["note"] = notes[1]
                    course["classRoom"] = notes[0]
                    if "未选中" in temp:
                        course["name"] = "[未选中]" + name
                    else:
                        course["name"] = name
                    course["day"] = i - 1
                    index = int(course["begin"] / 2)
                    if result[index]['classes'][i - 1] == '':  # 还没有课程的时候
                        result[index]['classes'][i - 1] = [course]
                    else:
                        isexist = False
                        for xx  in range(len(result[index]['classes'][i - 1])):
                            x = result[index]['classes'][i - 1][xx]
                            if x['name'] == course['name'] and x['classRoom'] == course['classRoom']:
                                '''该课程已经存在这一节，只是周数不同'''
                                result[index]['classes'][i - 1][xx]['weeks'].extend(course['weeks'])
                                isexist = True
                                break
                        if not isexist:
                            result[index]['classes'][i - 1].append(course)
        return result

    def get_json(self, doc):
        return json.loads(doc)

    def get_xml(self, doc):
        return xmltodict.parse(doc)

    def get_course_info(self, infos):
        result = {}
        temp = infos[0].split(" ")
        result['teacher'] = temp[0]
        result["weeks"] = self.getWeeks(temp[1]).split(",")
        begin, end = self.get_begin_end(temp[1])
        result["begin"] = begin
        result["num"] = end - begin
        if begin == 2 or begin == 8 or begin == 10:  # 计算偏移量
            py = -1
        else:
            py = 0
        result['py'] = py
        result['orgin'] = temp[1]
        return result

    def getWeeks(self, cc):  # 获取上课周数,
        "1-17周 四(3-4节)双"
        odd = 0  # 奇数
        even = 0  # 偶数
        if '双' in cc:
            even = 1
        if '单' in cc:
            odd = 1  #
        week = re.search(r"(\d+)-(\d+).?周", cc)
        weeks = ''
        if (week):
            begin = week.group(1)
            end = week.group(2)
            if even == 1:  # 偶数
                for x in range(int(begin), int(end) + 1):
                    if (x % 2 == 0):
                        weeks = weeks + str(x) + ','
            elif odd == 1:
                for x in range(int(begin), int(end) + 1):
                    if (x % 2 != 0):  # 奇数
                        weeks = weeks + str(x) + ','
            else:
                for x in range(int(begin), int(end) + 1):
                    weeks = weeks + str(x) + ','
        else:
            week = re.search(r"(\d+).?周", cc)
            if week:
                weeks += week.group(1) + ","
            else:
                for x in range(1, 18):
                    weeks = weeks + str(x) + ','
        return weeks[:-1]  # 返回周数序列

    def get_begin_end(self, strr):
        "1-17周 日(10-11节)"
        course = re.search(r"\[(\d+)-(\d+)\]", strr)
        if (course):
            return int(course.group(1)), int(course.group(2))
        else:
            return -1, -1
